@@ -27,7 +27,7 @@ G_BEGIN_DECLS
 
 GList      *e_categories_get_list (void);
 
-void        e_categories_add (const char *category, const char *color, const char *icon_file);
+void        e_categories_add (const char *category, const char *color, const char *icon_file, gboolean searchable);
 void        e_categories_remove (const char *category);
 
 gboolean    e_categories_exist (const char *category);
@@ -35,6 +35,7 @@ const char *e_categories_get_color_for (const char *category);
 void        e_categories_set_color_for (const char *category, const char *color);
 const char *e_categories_get_icon_file_for (const char *category);
 void        e_categories_set_icon_file_for (const char *category, const char *icon_file);
+gboolean    e_categories_is_searchable (const char *category);
 
 G_END_DECLS
 
